static const char *RcsId = "";
//+=============================================================================
//
// file :         LabmaxTop.cpp
//
// description :  C++ source for the LabmaxTop and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                LabmaxTop are implemented in this file.
//
// project :      TANGO Device Server
//
//
// Elettra-Sincrotrone Trieste S.C.p.A. di interesse nazionale
// Strada Statale 14 - km 163,5 in AREA Science Park
// 34149 Basovizza, Trieste ITALY
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The following table gives the correspondence
//	between commands and method name.
//
//  Command name     |  Method name
//	----------------------------------------
//  State            |  dev_state()
//  Status           |  dev_status()
//  Async_cmd        |  async_cmd()
//  Reset            |  reset()
//  Stats_start      |  stats_start()
//  Stats_stop       |  stats_stop()
//  Probe_zero       |  probe_zero()
//  GetPower_energy  |  get_power_energy()
//
//===================================================================


#include <tango.h>
#include <LabmaxTop.h>
#include <LabmaxTopClass.h>
#include "readthread.h"
#include "pwrthread.h"
#include <sys/time.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#ifndef SERVER_BUILDTIME
#define SERVER_BUILDTIME   __DATE__ " "  __TIME__
#endif

const char version_string[] = "$Build: @buildID@ " SERVER_BUILDTIME " $";


namespace LabmaxTop_ns
{

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::LabmaxTop(string &s)
// 
// description : 	constructor for simulated LabmaxTop
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
LabmaxTop::LabmaxTop(Tango::DeviceClass *cl,string &s)
:TANGO_BASE_CLASS(cl,s.c_str())
{
	init_device();
}

LabmaxTop::LabmaxTop(Tango::DeviceClass *cl,const char *s)
:TANGO_BASE_CLASS(cl,s)
{
	init_device();
}

LabmaxTop::LabmaxTop(Tango::DeviceClass *cl,const char *s,const char *d)
:TANGO_BASE_CLASS(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void LabmaxTop::delete_device()
{
	//	Delete device's allocated object
	abortflag = 1; //stops thread
	//usleep(20*pwr_rfsh*1000);
	usleep(700000);
	if (devinterface) {
		delete devinterface;
		devinterface = 0;
	}
	delete []usercmd;
	delete []userres;
	delete []trigger_param;
	delete []power_mode;
	delete []sub_unit;
	
//	delete pa_circ;


	delete mutex;

}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void LabmaxTop::init_device()
{
	INFO_STREAM << "LabmaxTop::LabmaxTop() create device " << device_name << " ver.:" << version_string << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();

	// connect to serialdevice 
	sz_debug = true;
//realtime
	rt_datacount = -1;
	rt_datastat = false;

	num_avg = DEF_NUM_AVG;

	devinterface = new Tango::DeviceProxy(interface);

	abortflag = 0;		//if<0 server is shutting down thread
	connection_status = -1;	//if<0 connection is lost
	wr_delay = 1;//was50;//was 500
	cmd_delay = 100;
	usercmd = new char[CMD_LEN];
	for(int k=0;k<CMD_LEN;k++)
		usercmd[k]=0;
//correction
	correction_on = true;
	corrected_energy = 0;
	if(correction.size()==0){
		correction_factor = DEF_CFACTOR;
		correction_offset = DEF_COFFSET;
	}
	else{
		vector<double>::iterator iter2 = correction.begin();
		correction_factor = ((double)*iter2);
		iter2++;
		correction_offset = ((double)*iter2);
	}

	userres = new char[RESP_LEN];
	memset(meas_unit,0,10);
	memset(userres, 0, RESP_LEN);
	memset(info_dev, 0, RESP_LEN);
	memset(info_status, 0, RESP_LEN);
	memset(info_error, 0, RESP_LEN);
	memset(info_probe, 0, RESP_LEN);
	memset(comm_intfc, 0, RESP_LEN);
	memset(comm_baud, 0, RESP_LEN);
	memset(comm_par, 0, SHORT_RESP_LEN);
	memset(comm_sbit, 0, SHORT_RESP_LEN);
	memset(comm_pace, 0, SHORT_RESP_LEN);
	memset(comm_serv, 0, SHORT_RESP_LEN);
	memset(probe_type, 0, SHORT_RESP_LEN);
	memset(probe_mod, 0, SHORT_RESP_LEN);
	memset(probe_num, 0, SHORT_RESP_LEN);
	memset(probe_resp, 0, SHORT_RESP_LEN);
	memset(probe_cdat, 0, SHORT_RESP_LEN);
	memset(probe_temp, 0, SHORT_RESP_LEN);
	memset(probe_diam, 0, SHORT_RESP_LEN);
	memset(sys_id, 0, SHORT_RESP_LEN);
	memset(sys_snum, 0, SHORT_RESP_LEN);
	memset(sys_cdat, 0, SHORT_RESP_LEN);
	memset(sys_type, 0, SHORT_RESP_LEN);
	remote_mode = false;
	zero = 0;
	range = 0;
	auto_range = true;

	gain_factor = 1;
	wavelength = 0;
	trigger_param = new char[RESP_LEN];
	power_mode = new char[4];
	memset(power_mode, 0, 4);
	sub_unit = new char[20];
	memset(sub_unit, 0, 20);

	wlen_correction = false;

	memset(trig_source_int,0,10);
	trig_int_lev = 0;
	memset(trig_ext_pos,0,10);
	trig_ext_del = 0;
	duration = 0;
	sampl_rate_pulse = 0;
	pulse_freq = 0;
	stats_on = false;
	mean = 0;
	sum = 0;
	nsamples = 0;
	sqsum = 0;
	std_dev = 0;
	max = 0;
	min = 0;
	memset(trigger_param, 0, RESP_LEN);
	memset(info_comm, 0, RESP_LEN);
	memset(wave_list, 0, RESP_LEN);
	memset(old_wave_list, 0, RESP_LEN);
	//map associates every meas unit from sub_unit to the right multiplier
	multipliers["W"] = 1;
	multipliers["mW"] = 1.0E+3;
	multipliers["uW"] = 1.0E+6;
	multipliers["nW"] = 1.0E+9;
	multipliers["J"] = 1;
	multipliers["mJ"] = 1.0E+3;
	multipliers["uJ"] = 1.0E+6;
	multipliers["nJ"] = 1.0E+9;
	multipliers["J/cm2"] = 1;
	multipliers["mJ/cm2"] = 1.0E+3;
	multipliers["uJ/cm2"] = 1.0E+6;
	multipliers["nJ/cm2"] = 1.0E+9;
	multipliers["W/cm2"] = 1;
	multipliers["mW/cm2"] = 1.0E+3;
	multipliers["uW/cm2"] = 1.0E+6;
	multipliers["nW/cm2"] = 1.0E+9;

//	pa_circ  = new Tango::DevVarDoubleArray();
//	pa_circ->length(PA_CIRC_LEN);
	for(int k=0;k<PA_CIRC_LEN;k++){
//		(*pa_circ)[k] = 0;
		val_double[k] = 0;
	}
	p_circ = 0;
	nrec = 0;
	pause_read = false;

	// check if serialdevice is responding 
	mutex = new omni_mutex();


	readloop = new readthread(this);
	readloop->start();
	//pwrloop = new pwrthread::pwrthread(this);
	//pwrloop->start();
}


//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void LabmaxTop::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("Interface"));
	dev_prop.push_back(Tango::DbDatum("Debug_mode"));
	dev_prop.push_back(Tango::DbDatum("Pwr_rfsh"));
	dev_prop.push_back(Tango::DbDatum("Correction"));
/*	dev_prop.push_back(Tango::DbDatum("RTBufferdevice"));
	dev_prop.push_back(Tango::DbDatum("RTLabmaxtopdevice"));*/
	dev_prop.push_back(Tango::DbDatum("RTSleep"));
	dev_prop.push_back(Tango::DbDatum("RTArraysize"));

	//	Call database and extract values
	//--------------------------------------------
	if (Tango::Util::instance()->_UseDb==true)
		get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	LabmaxTopClass	*ds_class =
		(static_cast<LabmaxTopClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize Interface from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  interface;
	else {
		//	Try to initialize Interface from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  interface;
	}
	//	And try to extract Interface value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  interface;

	//	Try to initialize Debug_mode from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  debug_mode;
	else {
		//	Try to initialize Debug_mode from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  debug_mode;
	}
	//	And try to extract Debug_mode value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  debug_mode;

	//	Try to initialize Pwr_rfsh from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  pwr_rfsh;
	else {
		//	Try to initialize Pwr_rfsh from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  pwr_rfsh;
	}
	//	And try to extract Pwr_rfsh value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  pwr_rfsh;

	//	Try to initialize Correction from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  correction;
	else {
		//	Try to initialize Correction from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  correction;
	}
	//	And try to extract Correction value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  correction;

	//	Try to initialize RTBufferdevice from class property
/*	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTBufferdevice;
	else {
		//	Try to initialize RTBufferdevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTBufferdevice;
	}
	//	And try to extract RTBufferdevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTBufferdevice;

	//	Try to initialize RTLabmaxtopdevice from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTLabmaxtopdevice;
	else {
		//	Try to initialize RTLabmaxtopdevice from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTLabmaxtopdevice;
	}
	//	And try to extract RTLabmaxtopdevice value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTLabmaxtopdevice;
*/
	//	Try to initialize RTSleep from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTSleep;
	else {
		//	Try to initialize RTSleep from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTSleep;
	}
	//	And try to extract RTSleep value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTSleep;

	//	Try to initialize RTArraysize from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  rTArraysize;
	else {
		//	Try to initialize RTArraysize from default device value
		def_prop = ds_class->get_default_device_property(dev_prop[i].name);
		if (def_prop.is_empty()==false)	def_prop  >>  rTArraysize;
	}
	//	And try to extract RTArraysize value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  rTArraysize;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}
//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void LabmaxTop::always_executed_hook()
{

}
//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_attr_hardware(vector<long> &attr_list)
{
	//DEBUG_STREAM << "LabmaxTop::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
	if(strlen(wave_list) > 0 && strcmp(wave_list,old_wave_list) != 0 && strstr(wave_list, ",") != NULL/*TODO: more then one wavelength, just to avoid savingwrong answers*/)
	{
		size_t pos_ini=0, pos_fin;
		string val;
		string resp(wave_list);
		DEBUG_STREAM << "LabmaxTop::read_attr_hardware updating wave list from '"<<old_wave_list<<"' to '" << wave_list << "'"<< endl;
		strcpy(old_wave_list,wave_list);
		Tango::DbDatum db_wavelen("Wavelength");	//TODO: attribute name, property name, ...
		db_wavelen << (short)1;	//1 attribute properties in "Wavelength"
		Tango::DbDatum db_values("values");	//TODO: attribute name, property name, ...
		Tango::DbData db_data_set;
		vector<string> str_values;
	    try {
			do
			{
				pos_fin = resp.find(",");
				if(pos_fin != string::npos)
				{
					val = resp.substr(pos_ini, pos_fin-pos_ini);
					resp = resp.substr(pos_fin+1, resp.length()-pos_fin-1 );
				}
				else if(resp.length() > 0)
				{
					val = resp;
				}
				//DEBUG_STREAM << "LabmaxTop::read_attr_hardware found wave len="<<val<< endl;
				str_values.push_back(val); 
			}while(pos_fin != string::npos);
			
			if(str_values.size() > 1)
			{
				db_values << str_values;
				db_data_set.push_back(db_wavelen);
				db_data_set.push_back(db_values);
				get_db_device()->put_attribute_property(db_data_set);
			}
	    }
	    catch(...)
	    {
	    	INFO_STREAM << "LabmaxTop::read_Wlen_list: ERROR PUTTING ATTRIBUTE PROPERTY" << endl;
	    };
	}
}
//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Num_avg
// 
// description : 	Extract real attribute values for Num_avg acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Num_avg(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Num_avg(Tango::Attribute &attr) entering... "<< endl;
	attr_Num_avg_read = &num_avg;
	attr.set_value(attr_Num_avg_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Num_avg
// 
// description : 	Write Num_avg attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Num_avg(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Num_avg(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Num_avg_write);
	if(attr_Num_avg_write>MAX_NUM_AVG)
			Tango::Except::throw_exception(
				(const char *)"Cant' write num_avg",
				(const char *)"Must be <=500",
				(const char *)"LabmaxTop::write_Num_avg()");	
	else
		num_avg = attr_Num_avg_write; 
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Correction_on
// 
// description : 	Extract real attribute values for Correction_on acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Correction_on(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Correction_on(Tango::Attribute &attr) entering... "<< endl;
	attr_Correction_on_read = &correction_on;
	attr.set_value(attr_Correction_on_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Correction_on
// 
// description : 	Write Correction_on attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Correction_on(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Correction_on(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Correction_on_write);
	if(attr_Correction_on_write){
		correction_on = true; 
	}
	else{
		correction_on = false; 
	}
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Rt_mode
// 
// description : 	Extract real attribute values for Rt_mode acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Rt_mode(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Rt_mode(Tango::Attribute &attr) entering... "<< endl;
	rt_mode = false;
	attr_Rt_mode_read = &rt_mode;
	attr.set_value(attr_Rt_mode_read, 1, 0, false);
	if(!rt_mode &&  get_status()=="Rt mode enabled")
		set_status("Reading");
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Rt_mode
// 
// description : 	Write Rt_mode attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Rt_mode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Rt_mode(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Rt_mode_write);

		Tango::Except::throw_exception(
			(const char *) "Error",
			(const char *) "Not Supported",
			(const char *) "LabmaxTop::write_Rt_mode()", Tango::ERR);			

}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Rt_data_stat
// 
// description : 	Extract real attribute values for Rt_data_stat acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Rt_data_stat(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Rt_data_stat(Tango::Attribute &attr) entering... "<< endl;
	attr_Rt_data_stat_read = &rt_datastat;
	attr.set_value(attr_Rt_data_stat_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Rt_data_count
// 
// description : 	Extract real attribute values for Rt_data_count acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Rt_data_count(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Rt_data_count(Tango::Attribute &attr) entering... "<< endl;
//	labmaxtop_get_counter(long *bunchnum, long *labmax_period, long *labmax_seq_num);
	attr_Rt_data_count_read = &rt_datacount;
	attr.set_value(attr_Rt_data_count_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Corrected_energy
// 
// description : 	Extract real attribute values for Corrected_energy acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Corrected_energy(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Corrected_energy(Tango::Attribute &attr) entering... "<< endl;

	if(correction_on)
		corrected_energy  =  (power_energy+ correction_offset)*correction_factor;
	else
		corrected_energy  =  power_energy;

	attr_Corrected_energy_read = &corrected_energy;
	attr.set_value(attr_Corrected_energy_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Correction_factor
// 
// description : 	Extract real attribute values for Correction_factor acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Correction_factor(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Correction_factor(Tango::Attribute &attr) entering... "<< endl;
	attr_Correction_factor_read = &correction_factor;
	attr.set_value(attr_Correction_factor_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Correction_factor
// 
// description : 	Write Correction_factor attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Correction_factor(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Correction_factor(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Correction_factor_write);
	correction_factor = attr_Correction_factor_write; 
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Correction_offset
// 
// description : 	Extract real attribute values for Correction_offset acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Correction_offset(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Correction_offset(Tango::Attribute &attr) entering... "<< endl;
	attr_Correction_offset_read = &correction_offset;
	attr.set_value(attr_Correction_offset_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Correction_offset
// 
// description : 	Write Correction_offset attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Correction_offset(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Correction_offset(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Correction_offset_write);
	correction_offset= attr_Correction_offset_write; 
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Pw_nrec
// 
// description : 	Extract real attribute values for Pw_nrec acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Pw_nrec(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Pw_nrec(Tango::Attribute &attr) entering... "<< endl;
	attr_Pw_nrec_read = &nrec;
	attr.set_value(attr_Pw_nrec_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Pw_idx
// 
// description : 	Extract real attribute values for Pw_idx acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Pw_idx(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Pw_idx(Tango::Attribute &attr) entering... "<< endl;
	attr_Pw_idx_read = &p_circ;
	attr.set_value(attr_Pw_idx_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Pw_inst
// 
// description : 	Extract real attribute values for Pw_inst acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Pw_inst(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Pw_inst(Tango::Attribute &attr) entering... "<< endl;
//	pw_circ_inst = (*pa_circ)[p_circ];
	pw_circ_inst = val_double[p_circ-1];
	attr_Pw_inst_read = &pw_circ_inst;
	attr.set_value(attr_Pw_inst_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Pw_buf
// 
// description : 	Extract real attribute values for Pw_buf acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Pw_buf(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Pw_buf(Tango::Attribute &attr) entering... "<< endl;
	attr_Pw_buf_read = &val_double[0];
	attr.set_value(attr_Pw_buf_read, PA_CIRC_LEN, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Duration
// 
// description : 	Extract real attribute values for Duration acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Duration(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Duration(Tango::Attribute &attr) entering... "<< endl;
	attr_Duration_read = &duration;
	attr.set_value(attr_Duration_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Duration
// 
// description : 	Write Duration attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Duration(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Duration(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Duration_write);
	string val = "CONF:STAT:BSIZ:PULS ";
	char buf[10];
	memset(buf, 0, 10);
	sprintf(buf, "%d",attr_Duration_write);
	val+=buf;	
	DEBUG_STREAM<<"writing duration: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	rate = (ID_PULSERATE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Pulse_freq
// 
// description : 	Extract real attribute values for Pulse_freq acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Pulse_freq(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Pulse_freq(Tango::Attribute &attr) entering... "<< endl;
	attr_Pulse_freq_read = &pulse_freq;
	attr.set_value(attr_Pulse_freq_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Gain_factor
// 
// description : 	Extract real attribute values for Gain_factor acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Gain_factor(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Gain_factor(Tango::Attribute &attr) entering... "<< endl;
	attr_Gain_factor_read = &gain_factor;
	attr.set_value(attr_Gain_factor_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Gain_factor
// 
// description : 	Write Gain_factor attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Gain_factor(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Gain_factor(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Gain_factor_write);
	string val = "CONF:GAIN:FACT ";
	char buf[10];
	memset(buf, 0, 10);
	sprintf(buf, "%f",attr_Gain_factor_write);
	val+=buf;	
	DEBUG_STREAM<<"writing gain_factor: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	rate = (ID_GAINFACT + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Area_corr
// 
// description : 	Extract real attribute values for Area_corr acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Area_corr(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Area_corr(Tango::Attribute &attr) entering... "<< endl;

}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Area_corr
// 
// description : 	Write Area_corr attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Area_corr(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Area_corr(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Aperture_area
// 
// description : 	Extract real attribute values for Aperture_area acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Aperture_area(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Aperture_area(Tango::Attribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Aperture_area
// 
// description : 	Write Aperture_area attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Aperture_area(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Aperture_area(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Out_volt_fscale
// 
// description : 	Extract real attribute values for Out_volt_fscale acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Out_volt_fscale(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Out_volt_fscale(Tango::Attribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Out_volt_fscale
// 
// description : 	Write Out_volt_fscale attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Out_volt_fscale(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Out_volt_fscale(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Wlen_correction
// 
// description : 	Extract real attribute values for Wlen_correction acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Wlen_correction(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Wlen_correction(Tango::Attribute &attr) entering... "<< endl;
	attr_Wlen_correction_read = &wlen_correction;
	attr.set_value(attr_Wlen_correction_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Wlen_correction
// 
// description : 	Write Wlen_correction attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Wlen_correction(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Wlen_correction(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Wlen_correction_write);
	wlen_correction = attr_Wlen_correction_write;
	string disp = wlen_correction?"CONF:WAVE:CORR ON":"CONF:WAVE:CORR OFF";
	string resp = asn_cmd(disp.c_str());
	//previous id
	rate = (ID_WAVECORR + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Wlen_list
// 
// description : 	Extract real attribute values for Wlen_list acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Wlen_list(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Wlen_list(Tango::Attribute &attr) entering... "<< endl;
	wave_list_ptr = wave_list;
	attr_Wlen_list_read = &wave_list_ptr;
	attr.set_value(attr_Wlen_list_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Wlen_list
// 
// description : 	Write Wlen_list attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Wlen_list(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Wlen_list(Tango::WAttribute &attr) entering... "<< endl;
	string cmd("CONF:WAVE:LIST ");
	attr.get_write_value(attr_Wlen_list_write);
	if(strlen(attr_Wlen_list_write) < RESP_LEN)
	{
		strncpy(wave_list, attr_Wlen_list_write, strlen(attr_Wlen_list_write));
		cmd+=wave_list;
		asn_cmd(cmd.c_str());
		//previous id
		rate = (ID_WAVELIST + ID_LAST - 1) % ID_LAST;
	}
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Trig_source_int
// 
// description : 	Extract real attribute values for Trig_source_int acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Trig_source_int(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Trig_source_int(Tango::Attribute &attr) entering... "<< endl;
	trig_source_int_ptr = trig_source_int;
	attr_Trig_source_int_read = &trig_source_int_ptr;
	attr.set_value(attr_Trig_source_int_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Trig_source_int
// 
// description : 	Write Trig_source_int attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Trig_source_int(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Trig_source_int(Tango::WAttribute &attr) entering... "<< endl;
	string cmd("TRIG:SOUR ");
	char tmp_trig_source_int[10];
	memset(tmp_trig_source_int,0,10);
	attr.get_write_value(attr_Trig_source_int_write);
	if(strlen(attr_Trig_source_int_write) < 9)
	{
		strncpy(tmp_trig_source_int, attr_Trig_source_int_write, strlen(attr_Trig_source_int_write));
		cmd+=tmp_trig_source_int;
		asn_cmd(cmd.c_str());
	}
	//previous id
	rate = (ID_TRIGSOUR + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Trig_int_lev
// 
// description : 	Extract real attribute values for Trig_int_lev acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Trig_int_lev(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Trig_int_lev(Tango::Attribute &attr) entering... "<< endl;
	attr_Trig_int_lev_read = &trig_int_lev;
	attr.set_value(attr_Trig_int_lev_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Trig_int_lev
// 
// description : 	Write Trig_int_lev attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Trig_int_lev(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Trig_int_lev(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Trig_int_lev_write);
	string val = "TRIG:LEV ";
	val+=attr_Trig_int_lev_write;
	DEBUG_STREAM<<"writing trig_int_lev: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_TRIGLEV + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Trig_ext_pos
// 
// description : 	Extract real attribute values for Trig_ext_pos acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Trig_ext_pos(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Trig_ext_pos(Tango::Attribute &attr) entering... "<< endl;
	//attr_Trig_ext_pos_read = &trig_ext_pos;
	trig_ext_pos_ptr = trig_ext_pos;
	attr_Trig_ext_pos_read = &trig_ext_pos_ptr;
	attr.set_value(attr_Trig_ext_pos_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Trig_ext_pos
// 
// description : 	Write Trig_ext_pos attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Trig_ext_pos(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Trig_ext_pos(Tango::WAttribute &attr) entering... "<< endl;
	string cmd("TRIG:SLOP ");
	char tmp_trig_ext_pos[10];
	memset(tmp_trig_ext_pos,0,10);
	attr.get_write_value(attr_Trig_ext_pos_write);
	if(strlen(attr_Trig_ext_pos_write) < 9)
	{
		strncpy(tmp_trig_ext_pos, attr_Trig_ext_pos_write, strlen(attr_Trig_ext_pos_write));
		cmd+=tmp_trig_ext_pos;
		asn_cmd(cmd.c_str());
	}
	//previous id
	rate = (ID_TRIGSLOP + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Trig_ext_del
// 
// description : 	Extract real attribute values for Trig_ext_del acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Trig_ext_del(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Trig_ext_del(Tango::Attribute &attr) entering... "<< endl;
	attr_Trig_ext_del_read = &trig_ext_del;
	attr.set_value(attr_Trig_ext_del_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Trig_ext_del
// 
// description : 	Write Trig_ext_del attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Trig_ext_del(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Trig_ext_del(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Trig_ext_del_write);
	string val = "TRIG:DEL ";
	val+=attr_Trig_ext_del_write;
	DEBUG_STREAM<<"writing trig_ext_del: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_TRIGDEL + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Batch_size_pulse
// 
// description : 	Extract real attribute values for Batch_size_pulse acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Batch_size_pulse(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Batch_size_pulse(Tango::Attribute &attr) entering... "<< endl;
	attr_Batch_size_pulse_read = &batch_size_pulse;
	attr.set_value(attr_Batch_size_pulse_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Batch_size_pulse
// 
// description : 	Write Batch_size_pulse attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Batch_size_pulse(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Batch_size_pulse(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Batch_size_pulse_write);
	string val = "CONF:STAT:BSIZ:PULS ";
	char buf[10];
	memset(buf, 0, 10);
	sprintf(buf, "%d",attr_Batch_size_pulse_write);
	val+=buf;	
	DEBUG_STREAM<<"writing batch_size_pulse: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_BATCHSIZE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Sampl_rate_pulse
// 
// description : 	Extract real attribute values for Sampl_rate_pulse acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Sampl_rate_pulse(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Sampl_rate_pulse(Tango::Attribute &attr) entering... "<< endl;
	attr_Sampl_rate_pulse_read = &sampl_rate_pulse;
	attr.set_value(attr_Sampl_rate_pulse_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Sampl_rate_pulse
// 
// description : 	Write Sampl_rate_pulse attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Sampl_rate_pulse(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Sampl_rate_pulse(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Sampl_rate_pulse_write);
	string val = "CONF:STAT:RATE:PULS ";
	char buf[20];
	memset(buf, 0, 20);
	sprintf(buf, "%d",attr_Sampl_rate_pulse_write);
	val+=buf;	
	DEBUG_STREAM<<"writing Sampl_rate_pulse: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_SAMPLERATEPULSE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Sampl_rate_time
// 
// description : 	Extract real attribute values for Sampl_rate_time acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Sampl_rate_time(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Sampl_rate_time(Tango::Attribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Sampl_rate_time
// 
// description : 	Write Sampl_rate_time attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Sampl_rate_time(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Sampl_rate_time(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Batch_size_time
// 
// description : 	Extract real attribute values for Batch_size_time acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Batch_size_time(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Batch_size_time(Tango::Attribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Batch_size_time
// 
// description : 	Write Batch_size_time attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Batch_size_time(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Batch_size_time(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Mean
// 
// description : 	Extract real attribute values for Mean acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Mean(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Mean(Tango::Attribute &attr) entering... "<< endl;


		attr_Mean_read = &mean;

	attr.set_value(attr_Mean_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Std_dev
// 
// description : 	Extract real attribute values for Std_dev acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Std_dev(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Std_dev(Tango::Attribute &attr) entering... "<< endl;

		attr_Std_dev_read = &std_dev;

	attr.set_value(attr_Std_dev_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Max
// 
// description : 	Extract real attribute values for Max acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Max(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Max(Tango::Attribute &attr) entering... "<< endl;
	attr_Max_read = &max;
	attr.set_value(attr_Max_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Min
// 
// description : 	Extract real attribute values for Min acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Min(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Min(Tango::Attribute &attr) entering... "<< endl;
	attr_Min_read = &min;
	attr.set_value(attr_Min_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Sub_unit
// 
// description : 	Extract real attribute values for Sub_unit acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Sub_unit(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Sub_unit(Tango::Attribute &attr) entering... "<< endl;
	attr_Sub_unit_read = &sub_unit;
	attr.set_value(attr_Sub_unit_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Power_mode
// 
// description : 	Extract real attribute values for Power_mode acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Power_mode(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Power_mode(Tango::Attribute &attr) entering... "<< endl;
	attr_Power_mode_read = &power_mode;
	attr.set_value(attr_Power_mode_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Power_mode
// 
// description : 	Write Power_mode attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Power_mode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Power_mode(Tango::WAttribute &attr) entering... "<< endl;
	char pmode[4];
	string cmd("CONF:MEAS ");
	memset(pmode, 0, 4);
	attr.get_write_value(attr_Power_mode_write);
	if(strlen(attr_Power_mode_write) < 4)
	{
		strncpy(pmode, attr_Power_mode_write, strlen(attr_Power_mode_write));
		cmd+=pmode;
		asn_cmd(cmd.c_str());
		string resp = asn_cmd("CONF:MEAS?");
		memset(power_mode, 0, 4);
		if(strlen(resp.c_str()) < 4)
			strncpy(power_mode, resp.c_str(), strlen(resp.c_str())-1);
		//previous id
		rate = (ID_POWMODE + ID_LAST - 1) % ID_LAST;
	}
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Info_comm
// 
// description : 	Extract real attribute values for Info_comm acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Info_comm(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Info_comm(Tango::Attribute &attr) entering... "<< endl;
	memset(info_comm, 0, RESP_LEN);
	strncat(info_comm, comm_intfc, strlen(comm_intfc));
	strncat(info_comm, comm_baud, strlen(comm_baud));
	strncat(info_comm, comm_par, strlen(comm_par));
	strncat(info_comm, comm_sbit, strlen(comm_sbit));
	strncat(info_comm, comm_pace, strlen(comm_pace));
	strncat(info_comm, comm_serv, strlen(comm_serv));
	
	info_comm_ptr = info_comm;
	attr_Info_comm_read = &info_comm_ptr;
	attr.set_value(attr_Info_comm_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Info_comm
// 
// description : 	Write Info_comm attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Info_comm(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Info_comm(Tango::WAttribute &attr) entering... "<< endl;
	Tango::Except::throw_exception(
		(const char *)"not yet implemented",
		(const char *)"not yet implemented",
		(const char *)"LabmaxTop::write_Info_comm()");
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Zero
// 
// description : 	Extract real attribute values for Zero acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Zero(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Zero(Tango::Attribute &attr) entering... "<< endl;
/*	attr_Zero_read = &zero;
	attr.set_value(attr_Zero_read, 1, 0, false);*/
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Zero
// 
// description : 	Write Zero attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Zero(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Zero(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Range
// 
// description : 	Extract real attribute values for Range acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Range(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Range(Tango::Attribute &attr) entering... "<< endl;
	attr_Range_read = &range;
	attr.set_value(attr_Range_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Range
// 
// description : 	Write Range attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Range(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Range(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Range_write);
	string val = "CONF:RANG:SEL ";
	char buf[20];
	memset(buf, 0, 20);
	sprintf(buf, "%E",(Tango::DevDouble)attr_Range_write);
	val+=buf;	
	DEBUG_STREAM<<"writing range: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_RANGE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Auto_range
// 
// description : 	Extract real attribute values for Auto_range acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Auto_range(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Auto_range(Tango::Attribute &attr) entering... "<< endl;
	attr_Auto_range_read = &auto_range;
	attr.set_value(attr_Auto_range_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Auto_range
// 
// description : 	Write Auto_range attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Auto_range(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Auto_range(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Auto_range_write);
	auto_range = attr_Auto_range_write;
	string disp = auto_range?"CONF:RANG:AUTO ON":"CONF:RANG:AUTO OFF";
	string resp = asn_cmd(disp.c_str());
	//previous id
	rate = (ID_AUTORANGE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Gain_comp
// 
// description : 	Extract real attribute values for Gain_comp acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Gain_comp(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Gain_comp(Tango::Attribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Gain_comp
// 
// description : 	Write Gain_comp attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Gain_comp(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Gain_comp(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Wavelength
// 
// description : 	Extract real attribute values for Wavelength acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Wavelength(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Wavelength(Tango::Attribute &attr) entering... "<< endl;
	attr_Wavelength_read = &wavelength;
	attr.set_value(attr_Wavelength_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Wavelength
// 
// description : 	Write Wavelength attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Wavelength(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Wavelength(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Wavelength_write);
	string val = "CONF:WAVE:WAVE ";
	char buf[20];
	memset(buf, 0, 20);
	sprintf(buf, "%d",(Tango::DevLong)attr_Wavelength_write);
	val+=buf;	
	DEBUG_STREAM<<"writing wavelength: "<<val.c_str()<<endl;
	string resp = asn_cmd(val.c_str());
	//previous id
	rate = (ID_WAVE + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Trigger_param
// 
// description : 	Extract real attribute values for Trigger_param acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Trigger_param(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Trigger_param(Tango::Attribute &attr) entering... "<< endl;
	memset(trigger_param, 0, RESP_LEN);
	attr_Trigger_param_read = &trigger_param;
	attr.set_value(attr_Trigger_param_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Trigger_param
// 
// description : 	Write Trigger_param attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Trigger_param(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Trigger_param(Tango::WAttribute &attr) entering... "<< endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Display_light
// 
// description : 	Extract real attribute values for Display_light acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Display_light(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Display_light(Tango::Attribute &attr) entering... "<< endl;
	attr_Display_light_read = &display_light;
	attr.set_value(attr_Display_light_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Display_light
// 
// description : 	Write Display_light attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Display_light(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Display_light(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Display_light_write);
	display_light = attr_Display_light_write;
	string disp = display_light?"DISP:BACK ON":"DISP:BACK OFF";
	string resp = asn_cmd(disp.c_str());
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Info_probe
// 
// description : 	Extract real attribute values for Info_probe acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Info_probe(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Info_probe(Tango::Attribute &attr) entering... "<< endl;
	memset(info_probe, 0, RESP_LEN);
	if(strlen(info_probe) + strlen(probe_type) < RESP_LEN)
		strncat(info_probe, probe_type, strlen(probe_type));
	if(strlen(info_probe) + strlen(probe_mod) < RESP_LEN)
		strncat(info_probe, probe_mod, strlen(probe_mod));
	if(strlen(info_probe) + strlen(probe_num) < RESP_LEN)
		strncat(info_probe, probe_num, strlen(probe_num));
	if(strlen(info_probe) + strlen(probe_resp) < RESP_LEN)
		strncat(info_probe, probe_resp, strlen(probe_resp));
	if(strlen(info_probe) + strlen(probe_cdat) < RESP_LEN)
		strncat(info_probe, probe_cdat, strlen(probe_cdat));
	if(strlen(info_probe) + strlen(probe_temp) < RESP_LEN)
		strncat(info_probe, probe_temp, strlen(probe_temp));
	if(strlen(info_probe) + strlen(probe_diam) < RESP_LEN)
		strncat(info_probe, probe_diam, strlen(probe_diam));
	
	info_probe_ptr = info_probe;
	attr_Info_probe_read = &info_probe_ptr;
	attr.set_value(attr_Info_probe_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Remote_mode
// 
// description : 	Extract real attribute values for Remote_mode acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Remote_mode(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Remote_mode(Tango::Attribute &attr) entering... "<< endl;
	string mode = remote_mode?"REMOTE":"LOCAL";
	//DEBUG_STREAM << "remote_mode read: "<< mode.c_str() << endl;
	attr_Remote_mode_read = &remote_mode;
	attr.set_value(attr_Remote_mode_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Remote_mode
// 
// description : 	Write Remote_mode attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Remote_mode(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Remote_mode(Tango::WAttribute &attr) entering... "<< endl;
	attr.get_write_value(attr_Remote_mode_write);
	remote_mode = attr_Remote_mode_write;
	if(remote_mode)
		asn_cmd("SYST:REM");
	else
		asn_cmd("SYST:LOC");
	string mode = remote_mode?"REMOTE":"LOCAL";
	DEBUG_STREAM << "remote_mode write: "<< mode.c_str() << endl;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Meas_unit
// 
// description : 	Extract real attribute values for Meas_unit acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Meas_unit(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Meas_unit(Tango::Attribute &attr) entering... "<< endl;
	meas_unit_ptr = meas_unit;
	attr_Meas_unit_read = &meas_unit_ptr;
	attr.set_value(attr_Meas_unit_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::write_Meas_unit
// 
// description : 	Write Meas_unit attribute values to hardware.
//
//-----------------------------------------------------------------------------
void LabmaxTop::write_Meas_unit(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "LabmaxTop::write_Meas_unit(Tango::WAttribute &attr) entering... "<< endl;
	char meas[4];
	string cmd("CONF:PUN ");
	memset(meas, 0, 4);
	attr.get_write_value(attr_Meas_unit_write);
	if(strlen(attr_Meas_unit_write) < 4)
		strncpy(meas, attr_Meas_unit_write, strlen(attr_Meas_unit_write));
	if (strncmp (meas,"W",1) == 0){
		cmd+=meas;
		asn_cmd(cmd.c_str());
	}
	else{
		if (strncmp (meas,"DBM",3) == 0){
			cmd+=meas;
			asn_cmd(cmd.c_str());
		}
		else{
			DEBUG_STREAM << "wrong meas unit "<<  endl;
			DEBUG_STREAM << "wrong meas unit "<<  endl;
			Tango::Except::throw_exception(
			(const char *)"wrong meas unit",
			(const char *)"wrong meas unit",
			(const char *)"LabmaxTop::write_Meas_unit()");
		}

	}
	string resp = asn_cmd("CONF:PUN?");
	memset(meas_unit, 0, 4);
	if(strlen(resp.c_str()) < 3)
		strncpy(meas_unit, resp.c_str(), strlen(resp.c_str())-1);
	//previous id
	rate = (ID_UNIT + ID_LAST - 1) % ID_LAST;
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Info_error
// 
// description : 	Extract real attribute values for Info_error acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Info_error(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Info_error(Tango::Attribute &attr) entering... "<< endl;
	if(get_state()==Tango::FAULT){
		memset(info_error, 0, RESP_LEN);
		strcpy(info_error, " lost connection");
	}
	if(get_state()==Tango::STANDBY){ 
		memset(info_error, 0, RESP_LEN);
		strcpy(info_error, " no new data available");
	}
	info_error_ptr = info_error;
	attr_Info_error_read = &info_error_ptr;
	attr.set_value(attr_Info_error_read, 1, 0, false);
}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Info_status
// 
// description : 	Extract real attribute values for Info_status acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Info_status(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Info_status(Tango::Attribute &attr) entering... "<< endl;
	info_status_ptr = info_status;
	attr_Info_status_read = &info_status_ptr;
	attr.set_value(attr_Info_status_read, 1, 0, false);
}


//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Power_energy
// 
// description : 	Extract real attribute values for Power_energy acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Power_energy(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Power_energy(Tango::Attribute &attr) entering... "<< endl;


	attr_Power_energy_read = &power_energy;
	attr.set_value(attr_Power_energy_read, 1, 0, false);

}

//+----------------------------------------------------------------------------
//
// method : 		LabmaxTop::read_Info_dev
// 
// description : 	Extract real attribute values for Info_dev acquisition result.
//
//-----------------------------------------------------------------------------
void LabmaxTop::read_Info_dev(Tango::Attribute &attr)
{
	//DEBUG_STREAM << "LabmaxTop::read_Info_dev(Tango::Attribute &attr) entering... "<< endl;

	memset(info_dev, 0, RESP_LEN);
	if(strlen(info_dev) + strlen(sys_id) < RESP_LEN)
		strncat(info_dev, sys_id, strlen(sys_id));
	if(strlen(info_dev) + strlen(sys_snum) < RESP_LEN)
		strncat(info_dev, sys_snum, strlen(sys_snum));
	if(strlen(info_dev) + strlen(sys_cdat) < RESP_LEN)
		strncat(info_dev, sys_cdat, strlen(sys_cdat));
	if(strlen(info_dev) + strlen(sys_type) < RESP_LEN)
		strncat(info_dev, sys_type, strlen(sys_type));

	info_dev_ptr = info_dev;
	attr_Info_dev_read = &info_dev_ptr;
	attr.set_value(attr_Info_dev_read, 1, 0, false);
}



//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::async_cmd
 *
 *	description:	method to execute "Async_cmd"
 *
 * @param	argin	command to be sent
 * @return	response if any
 *
 */
//+------------------------------------------------------------------
Tango::DevString LabmaxTop::async_cmd(Tango::DevString argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	DEBUG_STREAM << "LabmaxTop::async_cmd(): " <<argin<<" connection: "<< connection_status<<endl;

	//	Add your own code to control device here
	Tango::DevString	argout  ;
	string resp = asn_cmd(argin);
	argout  = new char[resp.length()+1];
	memset(argout, 0, resp.length()+1);
	strncpy(argout, resp.c_str(), resp.length());
	return argout;
	
	
}





//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::reset
 *
 *	description:	method to execute "Reset"
 *
 *
 */
//+------------------------------------------------------------------
void LabmaxTop::reset()
{
	DEBUG_STREAM << "LabmaxTop::reset(): entering... !" << endl;

	//	Add your own code to control device here
	string resp = asn_cmd("*RST");

}



//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::stats_start
 *
 *	description:	method to execute "Stats_start"
 *
 *
 */
//+------------------------------------------------------------------
void LabmaxTop::stats_start()
{
	DEBUG_STREAM << "LabmaxTop::stats_start(): entering... !" << endl;

	//	Add your own code to control device here
	string resp = asn_cmd("CONF:STAT:START");
	sum = 0;
	nsamples = 0;
	mean = 0;
	sqsum = 0;
	std_dev = 0;
	max = 0;
	min = -1;
	stats_on = true;

}

//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::stats_stop
 *
 *	description:	method to execute "Stats_stop"
 *
 *
 */
//+------------------------------------------------------------------
void LabmaxTop::stats_stop()
{
	DEBUG_STREAM << "LabmaxTop::stats_stop(): entering... !" << endl;

	//	Add your own code to control device here
	string resp = asn_cmd("CONF:STAT:STOP");
	stats_on = false;
//	mean = sum/nsamples;

}


//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::probe_zero
 *
 *	description:	method to execute "Probe_zero"
 *
 *
 */
//+------------------------------------------------------------------
void LabmaxTop::probe_zero()
{
	DEBUG_STREAM << "LabmaxTop::probe_zero(): entering... !" << endl;
	string resp = asn_cmd("CONF:ZERO");

	//	Add your own code to control device here

}




//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::asn_cmd
 *
 *	description:	method to execute "asn_cmd"
 *
 * @param	argin	command to be sent
 * @return	response if any
 *
 */
//+------------------------------------------------------------------
string LabmaxTop::asn_cmd(string argin)
{
	DEBUG_STREAM << "LabmaxTop::asn_cmd(): " <<argin<<" connection: "<< connection_status<<endl;

	//	Add your own code to control device here

	string s("");//contains the answer to the cmd if any
	long resu = 0; 
	//char	*argout  ;
	Tango::DevVarCharArray *new_array;
	string strout1, strout2;
	string input(argin);
	Tango::DeviceData Din, Dout;
	DEBUG_STREAM << "LabmaxTop::asn_cmd: argin is "<<argin.length()<<" char long"<<endl;
	new_array = new Tango::DevVarCharArray();
	new_array->length(argin.length()+2);
	//new_array->length(argin.length()+3);		//GZS
	
	for (int i=0;i<(int)argin.length();i++){
		(*new_array)[i] = argin[i];
		//DEBUG_STREAM<<argin[i];
	}
	(*new_array)[argin.length()] = '\x0d';
	(*new_array)[argin.length()+1] = '\x0a';
	//(*new_array)[argin.length()+2] = 0;		//GZS
	//DEBUG_STREAM << endl;
	
	Din << new_array;
	// write input to serial device
	DEBUG_STREAM << "LabmaxTop::asn_cmd: writing: " << argin << endl; 	
	mutex->lock();
	try {
		Dout = devinterface->command_inout("DevSerWriteChar",Din);
		Dout >> resu;
	}
	catch(Tango::DevFailed e) {	
		mutex->unlock();//otherwise this blocks the server until init
		connection_status = -1;
		INFO_STREAM << "LabmaxTop::asn_cmd: error writing on interface: " << interface << " err=" << e.errors[0].desc << endl;
		set_state(Tango::FAULT);
		set_status("Write errors from interface " + interface);			
/*		Tango::Except::re_throw_exception(e,
			(const char *) "serial device error",
			(const char *) "Cannot write to serial line",
			(const char *) "LabmaxTop::asn_cmd", Tango::ERR);	//20 06 to avoid abort*/	
	}
	DEBUG_STREAM<< "LabmaxTop::asn_cmd: written "<<resu<<" chars"<<endl;

	//TMP!!	abort_sleep(((double)wr_delay)/1000.0);
	
	string r_temp;
	bool readagain = false;
	// read confirm	
	// from serial device
	std::string::size_type j(input.find('?'));
	if(j != std::string::npos){
		DEBUG_STREAM << "LabmaxTop::asn_cmd: expected answer" << endl;
		omni_thread::sleep(0,100000000);
		strout1.clear();
		try {
			do{
				Dout = devinterface->command_inout("DevSerReadLine");
				Dout >> r_temp;
				//if((unsigned int)(r_temp.c_str()[r_temp.length()-2])==CR){
				std::size_t pos = r_temp.find(CR);
				if(pos == r_temp.length()-2){
					DEBUG_STREAM<<"LabmaxTop::asn_cmd: found CR"<<endl;
					readagain = false;}
				else{
					DEBUG_STREAM<<"LabmaxTop::asn_cmd: no terminator"<<endl;
					readagain = true;
				}
				strout1 += r_temp;
				if(connection_status == -1)
				{
					connection_status = 1;
					set_state(Tango::ON);
					set_status("Reading");					
				}
			}
			while(readagain);
			
		}
		catch(Tango::DevFailed e) {	
			mutex->unlock();
			if(input.find("FETC:") == string::npos)
			{
				connection_status = -1;
				set_state(Tango::FAULT);
				set_status(string("Read errors from interface ") + string(interface) + string(" err=") + string(e.errors[0].desc));
			}
			else
			{
				set_status("Data not updating, trigger missing?");
			}
			INFO_STREAM << "LabmaxTop::asn_cmd: error reading from interface: " << interface << endl;	
			Tango::Except::re_throw_exception(e,
				(const char *) "serial device error",
				(const char *) "Cannot read from serial line",
				(const char *) "LabmaxTop::asn_cmd", Tango::ERR);	//20 06 to avoid abort	
		}
		
	}
	string old_status=get_status();
	if((old_status=="Data not updating, trigger missing?") && (input.find("FETC:") != string::npos))
		set_status("Reading");
	DEBUG_STREAM<<"LabmaxTop::asn_cmd: received response='" << strout1 <<"' of "<<strout1.length()<<" chars" << endl;
	mutex->unlock();
	//new_array->length(0);
	/*for(int j=0;j<strout1.length();j++)
		DEBUG_STREAM<<(unsigned int)(strout1.c_str()[j])<<"/";
	DEBUG_STREAM<<endl;*/
/*	argout  = new char[strout1.length()+1];
	memset(argout, 0, strout1.length()+1);
	strncpy(argout, strout1.c_str(), strout1.length());
//	return argout;
	s = string(argout);
	delete [] argout;*/
	s = strout1;			
	return s;
	
	
}


#if 0
//+------------------------------------------------------------------
/**
 *	method:	LabmaxTop::get_power_energy
 *
 *	description:	method to execute "GetPower_energy"
 *
 * @param	argin	
 * @return	
 *
 */
//+------------------------------------------------------------------
Tango::DevVarDoubleArray *LabmaxTop::get_power_energy(const Tango::DevVarLongArray *argin)
{
	//	POGO has generated a method core with argout allocation.
	//	If you would like to use a static reference without copying,
	//	See "TANGO Device Server Programmer's Manual"
	//		(chapter : Writing a TANGO DS / Exchanging data)
	//------------------------------------------------------------
	Tango::DevVarDoubleArray	*argout  = new Tango::DevVarDoubleArray();
	DEBUG_STREAM << "LabmaxTop::get_power_energy(): entering... !" << endl;

	//	Add your own code to control device here
	long buf_idx[2], num_samples, data, ret;
	unsigned long bunchnumber;
	struct timeval buf_time,cur_time;
	
	gettimeofday(&cur_time,NULL);
	
	if (buf == NULL) {
		Tango::Except::throw_exception(
				(const char *) "Buffer RT error",
				(const char *) "Buffer RT not initialized",
				(const char *) "LabmaxTop::get_power_energy", Tango::ERR);	//20 06 to avoid abort	
	}
	ret = read_last(&data, &buf_time, &bunchnumber, buf);
	check_get_data_param(argin, buf_idx, &num_samples);
	argout->length(num_samples);

	/* get last value */
	//read_data_calib(long idx[2], double *data_buf, struct _buffer* buffer, double x2, double x1, double x0)
	if(correction_on)
		read_data_calib(0, buf_idx, &(*argout)[0], buf, 0, RT_MULT*correction_factor, correction_offset);
	else
		read_data_calib(0, buf_idx, &(*argout)[0], buf, 0, RT_MULT, 0);
	
	return argout;
}
#endif



//+------------------------------------------------------------------
//
//	method:			LabmaxTop::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void LabmaxTop::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.2); i++) {
		if (abortflag)
			break;
		omni_thread::sleep(0,200000000);
	}
}

}	//	namespace
