/*
 * pwrthread.cpp
 *
 *
 *
 */

#include "pwrthread.h"

#define QUEUE_LEN 100
#define NULL_READ_BEFORE_STANDBY 10
//abort/init period in us 
#define RFSH_MEAS   1000000 

pwrthread::pwrthread(LabmaxTop_ns::LabmaxTop *s):Tango::LogAdapter(s)
{ 
	t_read = s;
	interface_type = t_read->cmd_delay;
	power_array  = new Tango::DevVarDoubleArray();
	power_array->length(QUEUE_LEN);
	for(int k=0;k<QUEUE_LEN;k++){
		(*power_array)[k] = 0;
	}
}
		
pwrthread::~pwrthread() 
{
	DEBUG_STREAM << "pwrthread::~pwrthread(): destroying thread" << endl;
	delete power_array;
}

/* 
 *	pwrthread::run()
 *	Run  
 */		
void pwrthread::run(void *) 
{
	gettimeofday(&last_cache_update,NULL);
	// linux scheduler priority
/*	struct sched_param mysched;
	mysched.sched_priority = sched_get_priority_max(SCHED_RR) - 1;
	if( sched_setscheduler( 0, SCHED_FIFO, &mysched ) == -1 ) {
		cerr << "Launch server as root to gain maximum priority!" << endl;
	}
*/
	// omniorb priority
//	set_priority(PRIORITY_HIGH);
	
	while (!t_read->abortflag) {
	//	DEBUG_STREAM << "connection_status "<<t_read->connection_status<<endl;
		if(t_read->connection_status<0){
			//omni_thread::sleep(TIME_WAIT_RECONNECT, 0);//wait 5s for connection
			abort_sleep(TIME_WAIT_RECONNECT);
		
		}
		else{//connection available
			int flag;

				read_cont();
		}
	}
	
}

void pwrthread::read_cont() 
{
	char tsub_unit[20];
//	cout << "pwrthread read cont"<<endl;
	//DEBUG_STREAM << "sleeping "<<t_read->pwr_rfsh<<" ms"<<endl;
	//omni_thread::sleep(0, 1*t_read->pwr_rfsh);	//now micro
	abort_sleep(((double)t_read->pwr_rfsh)/1000.0);
	int standby_count = NULL_READ_BEFORE_STANDBY;

	time_t tim=time(NULL);
	tm *now=localtime(&tim);
	try{
		long readable=0;
		int bufferlen = QUEUE_LEN-1;
		string resp = t_read->asn_cmd("FETC:NREC?");////new
		sscanf (resp.c_str(),"%ld", &readable);
		DEBUG_STREAM << "pwrthread::read_cont: " << readable << " samples in buffer";
		t_read->nrec = readable;
		while(readable >=1 && bufferlen>0){//added = for last

			gettimeofday(&instant, NULL);

			if (abs((instant.tv_sec - last_cache_update.tv_sec)*1000000+ 
		   	(instant.tv_usec - last_cache_update.tv_usec)) > RFSH_MEAS*10) {
				DEBUG_STREAM<<"ABORT_INIT"<<endl;
				resp = t_read->asn_cmd("ABORT");
				gettimeofday(&last_cache_update,NULL);
				resp = t_read->asn_cmd("INIT");
			}

			standby_count = NULL_READ_BEFORE_STANDBY;
			t_read->set_state(Tango::ON);	
			t_read->pause_read = true;
			resp = t_read->asn_cmd("FETC:NEXT?");////new

			//DEBUG_STREAM<<"+";
			readable--;
			bufferlen--;
			string t, s, p, n;
			float tmp = 0;
			long pulse_per=1;
			long sequence=0;
			memset(tsub_unit, 0, 20);
			sscanf (resp.c_str(),"%E,%*s", &tmp);
			s = resp.substr(resp.find(",")+1);
			p = s.substr(s.find(",")+1);
			n = p.substr(p.find(",")+1);////new
			s = s.substr(0,s.find(","));
			p = p.substr(0, p.find(","));////new
			n = n.substr(0, n.find(","));////new
			long buh;
			sscanf (s.c_str(),"%s,%ld", tsub_unit,&buh);
			sscanf (p.c_str(),"%ld", &pulse_per);
			sscanf (n.c_str(),"%ld", &sequence);

			double t_pp = ((double)1000000/(double)pulse_per);
			DEBUG_STREAM<<"pwrthread::read_cont: tsub_unit:" << tsub_unit << " pulse_per:" << pulse_per<< " seq:"<<sequence<<endl;
			t_read->pulse_freq = (Tango::DevLong)((long)((t_pp)+0.5));
	
			if((strstr(t_read->power_mode,"W"))&&(strstr(t_read->info_probe,"PYRO"))){
				if(t_read->multipliers[string(tsub_unit)]==0){
					memset(tsub_unit, 0, 20);
					strncpy(tsub_unit, t_read->power_mode, strlen(t_read->power_mode));
					tpower_energy = tmp*t_read->pulse_freq;
				}
				else{
					tpower_energy = tmp*t_read->pulse_freq*t_read->multipliers[string(tsub_unit)];
				}
			}
			else{
				if(t_read->multipliers[string(tsub_unit)]==0){
					memset(tsub_unit, 0, 20);
					strncpy(tsub_unit, t_read->power_mode, strlen(t_read->power_mode));
					tpower_energy = tmp;
				}
				else{
					tpower_energy = tmp*t_read->multipliers[string(tsub_unit)];
				}
			}
			(*power_array)[bufferlen] = tpower_energy;
		//	(*(t_read->pa_circ))[t_read->p_circ] = tpower_energy;
			t_read->val_double[t_read->p_circ] = tpower_energy;
			t_read->p_circ = ((t_read->p_circ)+1)%PA_CIRC_LEN;

			memset(t_read->sub_unit, 0, 20);
			strncpy(t_read->sub_unit, tsub_unit, strlen(tsub_unit));
			t_read->power_energy  = tpower_energy;
			tduration = t_read->duration;
			//DEBUG_STREAM << now->tm_hour<<":"<<now->tm_min<<":"<<now->tm_sec<< ": power_energy: "<< tpower_energy <<" buf.pos:"<<t_read->p_circ<< endl;
			t_read->set_state(Tango::ON);		//TODO: remove

//statistics
/*			if((t_read->stats_on)){
				if(t_read->nsamples==tduration){
					t_read->sum = 0;
					t_read->nsamples = 0;
					t_read->mean = 0;
					t_read->sqsum = 0;
					t_read->std_dev = 0;
					t_read->max = 0;
					t_read->min = 0;
				}
				tmean = t_read->mean;
				tsum = t_read->sum;
				tsqsum = t_read->sqsum;
				tnsamples = t_read->nsamples;
	DEBUG_STREAM<<"sample n."<<tnsamples<<endl;
				tstd_dev = t_read->std_dev;
				tmin = t_read->min;
				tmax = t_read->max;
				tsum += tpower_energy;
				tnsamples += 1;
				tsqsum += (tpower_energy*tpower_energy);
				if(tnsamples != 0){
					tmean = tsum/tnsamples;
					tstd_dev = sqrt((tsqsum/tnsamples)-(tmean*tmean));
				}
				if(tmax-tpower_energy<0)tmax = tpower_energy;
				if(tpower_energy-tmin<0)tmin = tpower_energy;
				if(tnsamples==1)tmin = tpower_energy;
				t_read->max = tmax;
				t_read->min = tmin;
				t_read->mean = tmean;
				t_read->sum = tsum;
				t_read->sqsum = tsqsum;
				t_read->nsamples = tnsamples;
				t_read->std_dev = tstd_dev;
			}*/
		}///ok!!
		if(readable == 0){
			if(standby_count==0)
				t_read->set_state(Tango::STANDBY);
			else
				standby_count--;
		}
		t_read->pause_read = false;
	
	}
	catch(Tango::DevFailed e) {
		try{
			INFO_STREAM<<"pwrthread::read_cnt: dev_failed while read_cont"<<endl;
			int d = 0;
			string tst = t_read->asn_cmd("FETC:NREC?");
			sscanf (tst.c_str(),"%d", &d);
			if(d==0){
				DEBUG_STREAM << now->tm_hour<<":"<<now->tm_min<<":"<<now->tm_sec<<"- read_Power_energy(): data not available, last: " <<tpower_energy << endl; 			
				memset(t_read->info_error, 0, RESP_LEN);
				strcpy(t_read->info_error, "no new data available");
			}
		}	
		catch(Tango::DevFailed e) {
			INFO_STREAM<<"pwrthread::read_cnt: ERROR rereading FETC:NREC?"<<endl;
			t_read->connection_status = -1;
			t_read->set_state(Tango::STANDBY);	
			t_read->set_status("serial error");
		}
	}
	;
}
 
void pwrthread::read_rt() 
{
	DEBUG_STREAM<<"read_rt()"<<endl;
//	cout << "pwrthread read rt"<<endl;
	//omni_thread::sleep(1, 0);	//now micro
	abort_sleep(1);
}
 
 
//+------------------------------------------------------------------
//
//	method:			pwrthread::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void pwrthread::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.2); i++) {
		if (t_read->abortflag)
			break;
		omni_thread::sleep(0,200000000);
	}
}







