/*
 * readthread.cpp
 *
 *
 *
 */

#include "readthread.h"

#define QUEUE_LEN 100
#define NULL_READ_BEFORE_STANDBY 10
//abort/init period in us 
#define RFSH_MEAS   1000000 


readthread::readthread(LabmaxTop_ns::LabmaxTop *s):Tango::LogAdapter(s)
{ 
	t_read = s;
	interface_type = t_read->cmd_delay;
	power_array  = new Tango::DevVarDoubleArray();
	power_array->length(QUEUE_LEN);
	for(int k=0;k<QUEUE_LEN;k++){
		(*power_array)[k] = 0;
	}
}
		
readthread::~readthread() 
{
	DEBUG_STREAM << "readthread::~readthread(): destroying thread" << endl;
	power_array->length(0);
	delete power_array;
	
}

/* 
 *	readthread::run()
 *	Run  
 */		
void readthread::run(void *) 
{
	int ret;
	t_read->rate =12;	//TMP
	int counter = 0;
	
	while (!t_read->abortflag) {

		if(t_read->connection_status<0){
			try {
				if (t_read->devinterface->ping()) {
					//t_read->set_state(Tango::STANDBY);	
					t_read->set_status("thread connecting to interface " + t_read->interface);
					DEBUG_STREAM << "readthread::run(): thread connecting to interface " + t_read->interface << endl;
					
					Tango::DeviceData Din;
					Tango::DevLong flush = 2;  //flush input and output	
					Din << flush;
					try	{
						t_read->devinterface->command_inout("DevSerFlush",Din);
					}
					catch(Tango::DevFailed e) {
						string temp;
						temp = e.errors[0].desc;
						WARN_STREAM << "readthread::run(): error flushing serial device: " << temp << endl;
					}	
					abort_sleep(0.2);		
	
					string resp = t_read->asn_cmd("CONF:PUN?");
					DEBUG_STREAM << "readthread::run(): conf unit:  " <<resp.c_str() << endl;
					memset(t_read->meas_unit, 0, 4);
					if(resp.length() < 4)
						strncpy(t_read->meas_unit, resp.c_str(), strlen(resp.c_str()));
					{
						resp = t_read->asn_cmd("CONF:READ:SEND PRI,UNIT,PER,SEQ");////new
						resp = t_read->asn_cmd("CONF:READ:CONT LAST");
//						resp = t_read->asn_cmd("CONF:READ:CONT STRE");
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_type();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_mod();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_num();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_resp();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_cdat();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_temp();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_probe_diam();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_sys_id();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_sys_snum();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_sys_cdat();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_sys_type();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_intfc();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_baud();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_par();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_sbit();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_pace();
						omni_thread::sleep(0,50000000);     //50 ms)
						rfsh_comm_serv();
						omni_thread::sleep(0,50000000);     //50 ms)
						resp = t_read->asn_cmd("INIT");
						omni_thread::sleep(0,50000000);     //50 ms)
						resp = t_read->asn_cmd("CONF:READ:SEND?");
						DEBUG_STREAM << "readthread::run(): conf fmt:  " <<resp.c_str() << endl;
						std::string::size_type i(resp.find("PRI"));
						std::string::size_type j(resp.find("UNIT"));
						if((i != std::string::npos)&&(j != std::string::npos)){
							t_read->set_state(Tango::ON);	
							t_read->set_status("Connected to serial interface");
							t_read->connection_status = 1;
						}
						else{
							DEBUG_STREAM << "readthread::run(): retrying initialization " + t_read->interface << endl;
							Tango::DeviceData Din;
							Din << (Tango::DevLong)2;
							t_read->devinterface->command_inout("DevSerFlush",Din);
							//omni_thread::sleep(TIME_WAIT_RECONNECT,0);
							abort_sleep(TIME_WAIT_RECONNECT);
						}
					}
				}
			} 
			catch(Tango::DevFailed e) {
				t_read->set_state(Tango::FAULT);
				t_read->set_status("thread Failed to connect to interface " + t_read->interface);
				DEBUG_STREAM << "thread Failed to connect to interface " + t_read->interface << endl; 			
				//omni_thread::sleep(TIME_WAIT_RECONNECT,0);//10s between retries
				abort_sleep(TIME_WAIT_RECONNECT);	
			}
		}
		else{//connection available
			DEBUG_STREAM << "sleeping "<<t_read->pwr_rfsh<<"00 ms"<<endl;
			//omni_thread::sleep(0, t_read->pwr_rfsh*100000000);	//ok 20Hzs
			omni_thread::sleep(0, 150000000);	//150 ms
			//abort_sleep(t_read->pwr_rfsh*0.01);	//100 Hz

			{
				ret = read_cont();
				if(ret < 0)
				{
					INFO_STREAM << "readthread::run: exception in read_cont..." << endl;
					Tango::DeviceData Din2;
					Tango::DevLong flush = 2;	//IN & OUT
					Din2 << flush;
					t_read->devinterface->command_inout("DevSerFlush",Din2);
					abort_sleep(0.2);
					//continue;
				}
				counter++;
				if(counter %4!=2)
					continue;
//			cout << "readthread read cont"<<endl;
			string resp;
			//while(t_read->pause_read)abort_sleep(0.010);//omni_thread::sleep(0, t_read->pwr_rfsh*100);
			try{
			switch(t_read->rate%ID_LAST){
				case ID_ERR:
				rfsh_err();
				break;
				case ID_STAT:
				rfsh_stat();
				break;
				case ID_UNIT:
				rfsh_unit();
				break;
/*
				case ID_PULSERATE:
				rfsh_pulserate();
				break;

				case ID_BATCHSIZE:		//TODO: remove?
				rfsh_batchsize();
				break;
*/
				case ID_WAVECORR:
				rfsh_wavecorr();
				break;
				case ID_TRIGSOUR:
				rfsh_trigsour();
				break;
				case ID_SAMPLERATEPULSE:	//TODO: remove?
				rfsh_samplratepulse();
				break;
				case ID_AUTORANGE:
				rfsh_autorange();
				break;
				case ID_RANGE:
				rfsh_range();
				break;
/*
				case ID_GAINFACT:
				rfsh_gainfact();
				break;
*/
				case ID_WAVELIST:
				rfsh_wavelist();
				break;
				case ID_WAVE:
				rfsh_wave();
				break;
				case ID_TRIGLEV:
				rfsh_triglev();
				break;
				case ID_TRIGSLOP:
				rfsh_trigslop();
				break;
				case ID_TRIGDEL:
				rfsh_trigdel();
				break;
				case ID_POWMODE:
				rfsh_powmode();
				break;
			}
			} 
			catch(Tango::DevFailed e) {
				//omni_thread::sleep(0,1000);//	was 10000
				INFO_STREAM << "readthread::run: exception in some command..." << endl;
				abort_sleep(0.2);
			}
			t_read->rate++;
			}
		}
	}
	
}



////
void readthread::rfsh_pulserate(){	
	long d = 0;
	string resp = t_read->asn_cmd("CONF:STAT:BSIZ:PULS?");
	sscanf (resp.c_str(),"%ld", &d);
	t_read->duration = d;
}
////
void readthread::rfsh_samplratepulse(){	
	long s = 0;
	string resp = t_read->asn_cmd("CONF:STAT:RATE:PULS?");
	sscanf (resp.c_str(),"%ld", &s);
	t_read->sampl_rate_pulse = s;
}

////
void readthread::rfsh_gainfact(){
	float g = 0;
	string resp = t_read->asn_cmd("CONF:GAIN:FACT?");
	sscanf (resp.c_str(),"%E", &g);
	t_read->gain_factor = g;
}
////
void readthread::rfsh_wavecorr(){
	string resp = t_read->asn_cmd("CONF:WAVE:CORR?");
	std::string::size_type j(resp.find("ON"));
	if(j != std::string::npos){
		t_read->wlen_correction = true;
	}
	else{
		t_read->wlen_correction = false;
	}
}
////
void readthread::rfsh_wavelist(){
	memset(t_read->wave_list, 0, RESP_LEN);
	string resp = t_read->asn_cmd("CONF:WAVE:LIST?");
	if(strlen(resp.c_str()) < RESP_LEN)
		strncpy(t_read->wave_list, resp.c_str(), strlen(resp.c_str())-1);
}
////
void readthread::rfsh_trigsour(){
	string resp = t_read->asn_cmd("TRIG:SOUR?");
	memset(t_read->trig_source_int, 0, 10);
	if(strlen(resp.c_str()) < 10)
		strncpy(t_read->trig_source_int, resp.c_str(), strlen(resp.c_str()));
}
////
void readthread::rfsh_triglev(){
	int l = 0;
	string resp = t_read->asn_cmd("TRIG:LEV?");
	sscanf (resp.c_str(),"%d", &l);
	t_read->trig_int_lev = (Tango::DevShort)l;
}
////
void readthread::rfsh_trigslop(){
	string resp = t_read->asn_cmd("TRIG:SLOP?");
	memset(t_read->trig_ext_pos, 0, 10);
	if(strlen(resp.c_str()) < 10)
		strncpy(t_read->trig_ext_pos, resp.c_str(), strlen(resp.c_str()));
}
////
void readthread::rfsh_trigdel(){
	int d = 0;
	string resp = t_read->asn_cmd("TRIG:DEL?");
	sscanf (resp.c_str(),"%d", &d);
	t_read->trig_ext_del = (Tango::DevShort)d;
}
////
void readthread::rfsh_batchsize(){
	long b = 0;
	string resp = t_read->asn_cmd("CONF:STAT:BSIZ:PULS?");
	sscanf (resp.c_str(),"%ld", &b);
	t_read->batch_size_pulse = b;
}
////
void readthread::rfsh_powmode(){
	string resp = t_read->asn_cmd("CONF:MEAS?");
	memset(t_read->power_mode, 0, 4);
	if(strlen(resp.c_str()) < 4)
		strncpy(t_read->power_mode, resp.c_str(), strlen(resp.c_str())-1);
}
////
void readthread::rfsh_zero(){
	float z = 0;
	string resp = t_read->asn_cmd("CONF:ZERO?");
	sscanf (resp.c_str(),"%E", &z);
	t_read->zero = z;
}
////
void readthread::rfsh_range(){
	float r = 0;
	string resp = t_read->asn_cmd("CONF:RANG:SEL?");
	sscanf (resp.c_str(),"%E", &r);
	t_read->range = r;
}
////
void readthread::rfsh_autorange(){
	string resp = t_read->asn_cmd("CONF:RANG:AUTO?");
	std::string::size_type j(resp.find("ON"));
	if(j != std::string::npos){
		t_read->auto_range = true;
	}
	else{
		t_read->auto_range = false;
	}
}
////
void readthread::rfsh_wave(){
	long w = 0;
	string resp = t_read->asn_cmd("CONF:WAVE:WAVE?");
	sscanf (resp.c_str(),"%ld", &w);
	t_read->wavelength = w;
}
////
void readthread::rfsh_unit(){
	string resp = t_read->asn_cmd("CONF:PUN?");
	memset(t_read->meas_unit, 0, 4);
	if(strlen(resp.c_str()) < 4)
		strncpy(t_read->meas_unit, resp.c_str(), strlen(resp.c_str())-1);
}
////
void readthread::rfsh_err(){
	memset(t_read->info_error, 0, RESP_LEN);
	int numerr = 0;
	string resp = t_read->asn_cmd("SYST:ERR:COUNT?");
	sscanf (resp.c_str(),"%d", &numerr);
	char *tmp = new char[RESP_LEN];
	memset(tmp, 0, RESP_LEN);
	if(numerr>0){
		resp = t_read->asn_cmd("SYST:ERR:NEXT?");
		std::string::size_type j(resp.find(","));
		if(j != std::string::npos){
			resp = resp.substr(j+1);
		}
		if(RESP_LEN - strlen(t_read->info_error) > strlen(resp.c_str())+1)
			strncat(t_read->info_error, resp.c_str(), strlen(resp.c_str()));
		resp = t_read->asn_cmd("SYST:ERR:COUNT?");
		sscanf (resp.c_str(),"%d", &numerr);
	}
	delete []tmp;
}
////
void readthread::rfsh_stat(){
	string resp = t_read->asn_cmd("SYST:STAT?");
	int code = 0;
	sscanf (resp.c_str(),"%x", &code);
	memset(t_read->info_status, 0, RESP_LEN);
	for(int i=0;i<NUM_STAT;i++){
		if((code & hex_reg[i])>0 && RESP_LEN - strlen(t_read->info_status) > strlen(status_desc[i])+1){
			strncat(t_read->info_status, status_desc[i], strlen(status_desc[i]));
			strncat(t_read->info_status, "/", 1);
		}
	}
}
////
void readthread::rfsh_comm_intfc(){
	string resp = t_read->asn_cmd("SYST:COMM:INT?");
	memset(t_read->comm_intfc, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->comm_intfc, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->comm_intfc, "/", 1);
	}
}
////
void readthread::rfsh_comm_baud(){
	string resp = t_read->asn_cmd("SYST:COMM:SER:BAUD?");
	memset(t_read->comm_baud, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->comm_baud, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->comm_baud, "/", 1);
	}
}
////
void readthread::rfsh_comm_par(){
	string resp = t_read->asn_cmd("SYST:COMM:SER:PAR?");
	memset(t_read->comm_par, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->comm_par, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->comm_par, "/", 1);
	}
}
////
void readthread::rfsh_comm_sbit(){
	string resp = t_read->asn_cmd("SYST:COMM:SER:SBIT?");
	memset(t_read->comm_sbit, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->comm_sbit, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->comm_sbit, "/", 1);
	}
}
////
void readthread::rfsh_comm_pace(){
	string resp = t_read->asn_cmd("SYST:COMM:SER:PACE?");
	memset(t_read->comm_pace, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->comm_pace, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->comm_pace, "/", 1);
	}
}
////
void readthread::rfsh_comm_serv(){
	string resp = t_read->asn_cmd("SYST:COMM:SER:SERV?");
	memset(t_read->comm_serv, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN)
		strncpy(t_read->comm_serv, resp.c_str(), strlen(resp.c_str())-1);
//	strncat(t_read->comm_serv, "/", 1);
}
////

void readthread::rfsh_probe_type(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:TYPE?");
	memset(t_read->probe_type, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_type, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_type, "/", 1);
	}
}
////
void readthread::rfsh_probe_mod(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:MOD?");
	memset(t_read->probe_mod, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_mod, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_mod, "/", 1);
	}
}
void readthread::rfsh_probe_num(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:SNUM?");
	memset(t_read->probe_num, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_num, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_num, "/", 1);
	}
}
void readthread::rfsh_probe_resp(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:RESP?");
	memset(t_read->probe_resp, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_resp, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_resp, "/", 1);
	}
}
void readthread::rfsh_probe_cdat(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:CDAT?");
	memset(t_read->probe_cdat, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_cdat, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_cdat, "/", 1);
	}
}
void readthread::rfsh_probe_temp(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:TEMP?");
	memset(t_read->probe_temp, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->probe_temp, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->probe_temp, "/", 1);
	}
}
void readthread::rfsh_probe_diam(){
	string resp = t_read->asn_cmd("SYST:INF:PROB:DIAM?");
	memset(t_read->probe_diam, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN)
		strncpy(t_read->probe_diam, resp.c_str(), strlen(resp.c_str())-1);
//	strncat(t_read->probe_diam, "/", 1);
}
void readthread::rfsh_sys_id(){
	string resp = t_read->asn_cmd("*IDN?");
	memset(t_read->sys_id, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->sys_id, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->sys_id, "/", 1);
	}
}
void readthread::rfsh_sys_snum(){
	string resp = t_read->asn_cmd("SYST:INF:INST:SNUM?");
	memset(t_read->sys_snum, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->sys_snum, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->sys_snum, "/", 1);
	}
}
void readthread::rfsh_sys_cdat(){
	string resp = t_read->asn_cmd("SYST:INF:INST:CDAT?");
	memset(t_read->sys_cdat, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->sys_cdat, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->sys_cdat, "/", 1);
	}
}
void readthread::rfsh_sys_type(){
	string resp = t_read->asn_cmd("SYST:INF:INST:TYPE?");
	memset(t_read->sys_type, 0, RESP_LEN);
	if(strlen(resp.c_str()) < RESP_LEN-1)
	{
		strncpy(t_read->sys_type, resp.c_str(), strlen(resp.c_str())-1);
		strncat(t_read->sys_type, "/", 1);
	}
}



int readthread::read_cont() 
{
	char tsub_unit[64];
//	cout << "pwrthread read cont"<<endl;
	//DEBUG_STREAM << "sleeping "<<t_read->pwr_rfsh<<" ms"<<endl;
	//omni_thread::sleep(0, 1*t_read->pwr_rfsh);	//now micro
	//abort_sleep(((double)t_read->pwr_rfsh)/1000.0);
	int standby_count = NULL_READ_BEFORE_STANDBY;
	string resp;
	time_t tim=time(NULL);
	tm *now=localtime(&tim);
	try{

		{//added = for last

			gettimeofday(&instant, NULL);

			if (abs((instant.tv_sec - last_cache_update.tv_sec)*1000000+ 
		   	(instant.tv_usec - last_cache_update.tv_usec)) > RFSH_MEAS*10) {
				DEBUG_STREAM<<"ABORT_INIT"<<endl;
				resp = t_read->asn_cmd("ABORT");
				gettimeofday(&last_cache_update,NULL);
				resp = t_read->asn_cmd("INIT");
				return 0;
			}

			standby_count = NULL_READ_BEFORE_STANDBY;
			//t_read->set_state(Tango::ON);	
			t_read->pause_read = true;
			//resp = t_read->asn_cmd("FETC:NEXT?");////new
			resp = t_read->asn_cmd("FETC:ALL?");////new

			string t, s, p, n;
			float tmp = 0;
			long pulse_per=1;
			long sequence=0;
			memset(tsub_unit, 0, 64);
			sscanf (resp.c_str(),"%E,%*s", &tmp);
			s = resp.substr(resp.find(",")+1);
			p = s.substr(s.find(",")+1);
			n = p.substr(p.find(",")+1);////new
			s = s.substr(0,s.find(","));
			p = p.substr(0, p.find(","));////new
			n = n.substr(0, n.find(","));////new
			long buh;
			sscanf (s.c_str(),"%s,%ld", tsub_unit,&buh);
			sscanf (p.c_str(),"%ld", &pulse_per);
			sscanf (n.c_str(),"%ld", &sequence);

			double t_pp = ((double)1000000/(double)pulse_per);
			DEBUG_STREAM<<"readthread::read_cont: tsub_unit:" << tsub_unit << " pulse_per:" << pulse_per<< " seq:"<<sequence<<endl;
			t_read->pulse_freq = (Tango::DevLong)((long)((t_pp)+0.5));
	
			if((strstr(t_read->power_mode,"W"))&&(strstr(t_read->info_probe,"PYRO"))){
				if(t_read->multipliers[string(tsub_unit)]==0){
					memset(tsub_unit, 0, 64);
					strncpy(tsub_unit, t_read->power_mode, strlen(t_read->power_mode));
					tpower_energy = tmp*t_read->pulse_freq;
					DEBUG_STREAM<<"readthread::read_cont: W & PYRO & mult=0 -> tmp(" << tmp << ")*pulse_freq(" << t_read->pulse_freq << ")=tpower_energy("<<tpower_energy<<")"<<endl;
				}
				else{
					tpower_energy = tmp*t_read->pulse_freq*t_read->multipliers[string(tsub_unit)];
					DEBUG_STREAM<<"readthread::read_cont: W & PYRO & mult!=0 -> tmp(" << tmp << ")*pulse_freq(" << t_read->pulse_freq << ")*mult("<<t_read->multipliers[string(tsub_unit)]<<"=tpower_energy("<<tpower_energy<<")"<<endl;
				}
			}
			else{
				if(t_read->multipliers[string(tsub_unit)]==0){
					memset(tsub_unit, 0, 64);
					strncpy(tsub_unit, t_read->power_mode, strlen(t_read->power_mode));
					tpower_energy = tmp;
					DEBUG_STREAM<<"readthread::read_cont: !(W & PYRO) & mult=0 -> tmp(" << tmp << ")=tpower_energy("<<tpower_energy<<")"<<endl;
				}
				else{
					tpower_energy = tmp*t_read->multipliers[string(tsub_unit)];
					DEBUG_STREAM<<"readthread::read_cont: !(W & PYRO) & mult!=0 -> tmp(" << tmp << ")*mult("<<t_read->multipliers[string(tsub_unit)]<<")=tpower_energy("<<tpower_energy<<")"<<endl;
				}
			}
			t_read->val_double[t_read->p_circ] = tpower_energy;
//			t_read->p_circ = ((t_read->p_circ)+1)%PA_CIRC_LEN;
			
			memset(t_read->sub_unit, 0, 64);
			strncpy(t_read->sub_unit, tsub_unit, strlen(tsub_unit));
			t_read->power_energy  = tpower_energy;
			tduration = t_read->duration;
			//DEBUG_STREAM << now->tm_hour<<":"<<now->tm_min<<":"<<now->tm_sec<< ": power_energy: "<< tpower_energy <<" buf.pos:"<<t_read->p_circ<< endl;
			//t_read->set_state(Tango::ON);		//TODO: remove
			
//statistics
			if((t_read->stats_on)){
				//std/mean algorithm due to Knuth
				int ind=t_read->p_circ;
				int count=0;
				double mean=0, std=0;
				int size = (t_read->num_avg > PA_CIRC_LEN) ? PA_CIRC_LEN : t_read->num_avg;
				double delta = 0;
				double M2 = 0;
				double correction_factor = t_read->correction_on ? t_read->correction_factor : 1;
				double correction_offset = t_read->correction_on ? t_read->correction_offset : 0;
				while(count < size)
				{
					count++;
					delta =  (correction_factor * t_read->val_double[ind] + correction_offset) - mean;
					mean = mean + delta/count;
					M2 = M2 + delta*((correction_factor * t_read->val_double[ind] + correction_offset) - mean);  // This expression uses the new value of mean
					ind = (ind - 1 + PA_CIRC_LEN) % PA_CIRC_LEN;
				}
				std = M2/(size - 1);
				t_read->mean = mean;
				//*(mydevice->attr_Freq1Variance_read) = std;
				t_read->std_dev = (std > 0) ? sqrt(std) : 0;
				if(tmax-correction_factor *tpower_energy+ correction_offset<0)tmax = (correction_factor *tpower_energy+ correction_offset);
				if(correction_factor *tpower_energy+ correction_offset-tmin<0 || t_read->min==-1)tmin = (correction_factor *tpower_energy+ correction_offset);
				if(tnsamples==1)tmin = (correction_factor *tpower_energy+ correction_offset);
				t_read->max = tmax;
				t_read->min = tmin;
			}
			t_read->p_circ = ((t_read->p_circ)+1)%PA_CIRC_LEN;
			DEBUG_STREAM<<"readthread::read_cont: exiting..."<<endl;
		}///ok!!
	
	}
	catch(Tango::DevFailed e) {
		INFO_STREAM<<"readthread::read_cnt: dev_failed while read_cont"<<endl;
		return -1;
	}
	return 0;
}


//+------------------------------------------------------------------
//
//	method:			readthread::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void readthread::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.2); i++) {
		if (t_read->abortflag)
			break;
		omni_thread::sleep(0,200000000);
	}

}









