/*
 * readthread.h
 *
 */ 

#ifndef readTHREAD_H
#define readTHREAD_H

#include <omnithread.h>
#include <tango.h>
#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>    
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <arpa/inet.h>
#include <signal.h>
#include <netdb.h>    
#include "LabmaxTop.h"

static int hex_reg[NUM_STAT] = {
0x00001,
0x00002,
0x00004,
0x00008,
0x00010,
0x00020,
0x00040,
0x00080,
0x00100,
0x00200,
0x00400,
0x00800,
0x01000
};

static char* status_desc[NUM_STAT] = {
"Battery is low",
"Sensor damage temperature is exceeded",
"Valid sensor is attached",
"USB flash drive is attached",
"Pyroelectic autoset is in progress",
"Batch data collection is in progress",
"Trend data collection is in progress",
"Meter power is on",
"Future use",
"Data logging to flash active",
"Data logging to in-memory FIFO archive",
"Remote mode enabled",
"Host file copy in progress"
};

class readthread : public omni_thread, public Tango::LogAdapter{	
	private:
		LabmaxTop_ns::LabmaxTop *t_read;		
		int interface_type;
		void abort_sleep(double time);
		int read_cont();
	struct timeval instant;
	struct timeval last_cache_update;
	Tango::DevVarDoubleArray *power_array;
		
	public:
		
		readthread(LabmaxTop_ns::LabmaxTop *s);
		
		~readthread();
					
	protected:
		void run(void *);	
	Tango::DevDouble tmean;
	Tango::DevDouble tsum;
	Tango::DevDouble tsqsum;
	Tango::DevLong tnsamples;
	Tango::DevDouble tstd_dev;
	Tango::DevDouble tmin;
	Tango::DevDouble tmax;
	Tango::DevString tsub_unit;
	Tango::DevDouble tpower_energy;
	Tango::DevLong tduration;
	
	void rfsh_pulserate();	
	void rfsh_gainfact();		
	void rfsh_wavecorr();	
	void rfsh_wavelist();	
	void rfsh_trigsour();	
	void rfsh_triglev();	
	void rfsh_trigslop();	
	void rfsh_trigdel();	
	void rfsh_batchsize();	
	void rfsh_powmode();	
	void rfsh_zero();	
	void rfsh_range();	
	void rfsh_autorange();		
	void rfsh_wave();	
	void rfsh_unit();	
	void rfsh_err();	
	void rfsh_stat();	
	void rfsh_comm_intfc();
	void rfsh_comm_baud();
	void rfsh_comm_par();
	void rfsh_comm_sbit();
	void rfsh_comm_pace();
	void rfsh_comm_serv();
	void rfsh_samplratepulse();	

	void rfsh_probe_type();
	void rfsh_probe_mod();
	void rfsh_probe_num();
	void rfsh_probe_resp();
	void rfsh_probe_cdat();
	void rfsh_probe_temp();
	void rfsh_probe_diam();
	void rfsh_sys_id();
	void rfsh_sys_snum();
	void rfsh_sys_cdat();
	void rfsh_sys_type();			
};	



#endif

/* EOF */
